﻿/**************************************
 * 定义
 *************************************/

 //鼠标状态
var Mouse = {
	Normal: 0,
	Hover : 1,
	Pressed : 2,
}

//文本显示方式
var TextAlign = {
	LEFT : 0x00000000,
	CENTER : 0x00000001,
	RIGHT : 0x00000002,
	TOP : 0x00000000,
	VCENTER : 0x00000004,
	BOTTOM : 0x00000008,
	SINGLELINE : 0x00000020,
	NOCLIP : 0x00000100,
	CALCRECT : 0x00000400,
	END_ELLIPSIS : 0x00008000,
	NOPREFIX : 0x00000800
}


function Size(w, h){
	this.Width = w;
	this.Height = h;
}

/**************************************
 * 按钮
 *************************************/
function Button(x, y, w, h, img_normal, img_hover, img_pressed, func, tiptext) {
	this.X = x;
	this.Y = y;
	this.Width = w;
	this.Height = h;
	this.Visible = true;
	this.Func = func || function(){};
	
	this.ImgNormal = img_normal ? gdi.Image(img_normal) : null;
	this.ImgHover = img_hover ? gdi.Image(img_hover) : this.ImgNormal;
	this.ImgPressed = img_pressed ? gdi.Image(img_pressed) : this.ImgNormal;

	this.Image = this.ImgNormal;
	
	this.State = Mouse.Normal;
	this.IsChecked = false;
	this.Tiptext = tiptext || "";	
	this.Tooltip = window.CreateTooltip();

	this.MouseX = -1;
	this.MouseY = -1;

	this.Paint = function(gr) {
		if (!this.Visible) return;

		switch (this.State) {
			case Mouse.Normal:
				this.Image = this.ImgNormal;
				break;
			case Mouse.Hover:
				this.Image = this.ImgHover;
				break;
			case Mouse.Pressed:
				this.Image = this.ImgPressed;
				break;
			default:
				break;
		}

		this.Image && gr.DrawImage(this.Image, this.X, this.Y, this.Width, this.Height, 0, 0, this.Image.Width, this.Image.Height);
	}

	this.MouseMove = function(x, y) {
		if (!this.Visible) return false;	

		if (x > this.X && x < this.X + this.Width 
			&& y > this.Y && y < this.Y + this.Height) {				
			if (this.State == Mouse.Normal) {
				this.State = Mouse.Hover;
				this.Tooltip.Text = this.Tiptext;
				this.Tooltip.Activate();
				this.Repaint();
			}	
			
			this.MouseX = x;
			this.MouseY = y;
			return true;
		} else {
			this.Reset();
			return false;
		}
	}

	this.MouseDown = function(x, y) {
		if (!this.Visible) return false;

		if (this.State == Mouse.Hover) {
			this.State = Mouse.Pressed;
			this.MouseX = x;
			this.MouseY = y;
			this.Repaint();
			return true;
		} else {
			return false;
		}
	}

	this.MouseUp = function() {
		if (!this.Visible) return false;

		if (this.MouseX > this.X && 
			this.MouseX < this.X + this.Width && 
			this.MouseY > this.Y && 
			this.MouseY < this.Y + this.Height) {
				this.State = Mouse.Hover;
				this.Func();
				this.Repaint();
				return true;
		}else{
			return false;
		}

		
		// if (this.State == Mouse.Pressed) {
		// 	this.State = Mouse.Hover;		
		// 	this.Func();
		// 	return true;
		// } else{
		// 	return false;
		// } 
	}

	this.Reset = function() {
		this.State = Mouse.Normal;
		this.Tooltip.Deactivate();
		this.Repaint();
		this.MouseX = -1;
		this.MouseY = -1;	
	}

	this.Repaint = function() {
		window.RepaintRect(this.X, this.Y, this.Width, this.Height);
	}

	this.SetVisible = function(visible){
		if (this.Visible != visible){
			this.Visible = visible;
			this.Reset();	
			this.Repaint();	
		}
	}

	this.SetChecked = function(isChecked){
		if (this.IsChecked != isChecked){
			this.IsChecked = isChecked;
			this.Repaint();
		}
	}

	this.SetImage = function(img_normal, img_hover, img_pressed){
		this.ImgNormal = img_normal ? gdi.Image(img_normal) : null;
		this.ImgHover = img_hover ? gdi.Image(img_hover) : this.ImgNormal;
		this.ImgPressed = img_pressed ? gdi.Image(img_pressed) : this.ImgNormal;
		this.Repaint();
	}

	this.SetTipText = function(tip){
		this.Tiptext = tip || "";	
	}
}

/**************************************
 * 滑动条
 *************************************/
function Slider(x, y, w, h, frame, overaly, seeker, frame_height, seeker_width, seeker_height) {
	this.X = x;
	this.Y = y;
	this.Width = w;
	this.Height = h;
	this.Enabled = true;

	this.State = Mouse.Normal;
	this.Value = 0;
	this.MinValue = 0;
	this.MaxValue = 100;

	this.Frame = frame;
	this.Overaly = overaly;
	this.Seeker = seeker;

	this.FrameHeight = frame_height;
	this.SeekerWidth = seeker_width;
	this.SeekerHeight = seeker_height;

	this.SetValue = function(value) {
		if (this.State != Mouse.Pressed){ 
			if (value < this.MinValue) value = this.MinValue;
			if (value > this.MaxValue) value = this.MaxValue;
			this.Value = value;
			this.Repaint();
		}
	}

	this.Paint = function(gr) {
		if (this.Enabled) {
			var pos = (this.Value / (this.MaxValue - this.MinValue) * this.Width) | 0;			
			var frame_y = Math.floor((this.Height - this.FrameHeight)/2) + this.Y;

			gr.DrawImage(this.Frame, this.X, frame_y, this.Width, this.FrameHeight, 0, 0, this.Frame.Width, this.Frame.Height, 0);
			gr.DrawImage(this.Overaly, this.X, frame_y, pos, this.FrameHeight, 0, 0, this.Overaly.Width, this.FrameHeight, 0);

			var seeker_x = Math.floor(this.X + pos - (this.SeekerWidth / 2));
			var seeker_y = Math.floor((this.Height - this.SeekerHeight)/2) + this.Y;
			gr.DrawImage(this.Seeker, seeker_x, seeker_y, this.SeekerWidth, this.SeekerHeight, 0, 0, this.SeekerWidth, this.SeekerHeight, 0);
		}
	}

	this.MouseDown = function(x, y) {
		if (this.State == Mouse.Hover) {
			this.State = Mouse.Pressed;
			if (x > this.X && x < this.X + this.Width) {
				var pos = x - this.X;
				this.Value = (pos * (this.MaxValue - this.MinValue) / this.Width) | 0;
				this.Repaint();
			}
			return true;
		} else return false;
	}

	this.MouseMove = function(x, y) {
		var value;
		if (this.State == Mouse.Pressed) {
			if (x <= this.X) {
				value = this.MinValue;
			}
			else {
				if (x >= this.X + this.Width){
					value = this.MaxValue;
				}
				else {
					var pos = x - this.X;
					value = (pos * (this.MaxValue - this.MinValue) / this.Width) | 0;
				}
			}
			this.Value = value;
			this.Repaint();
			return true;
		} else {
			if (x > this.X - this.SeekerWidth / 2 
				&& x < this.X + this.Width + this.SeekerWidth / 2 
				&& y > this.Y 
				&& y < this.Y + h && this.Enabled) {
				this.State = Mouse.Hover;
				return true;
			} else {
				this.State = Mouse.Normal;
			}
		}
		return false;
	}

	this.MouseUp = function() {
		if (this.State == Mouse.Pressed) {
			this.State = Mouse.Normal;
			return true;
		}
		return false;
	}

	this.MouseWheel = function(step, valueStep) {
		if (this.State == Mouse.Hover) {
			var newValue = this.Value + (step * valueStep);
			if (newValue < this.MinValue) newValue = this.MinValue;
			if (newValue > this.MaxValue) newValue = this.MaxValue;
			this.SetValue(newValue);
			return true;
		}
		return false;
	}

	this.Repaint = function() {
		window.RepaintRect(this.X - this.SeekerWidth/2, this.Y, this.Width + this.SeekerWidth, this.Height);
	}
}

/**************************************
 * 提示框
 *************************************/
// function ToolTip(x, y, text, font, text_color, bg_image) {
// 	this.X = x;
// 	this.Y = y;
// 	this.Width = 0;
// 	this.Height = 0
// 	this.Text = text;
// 	this.Font = font;
// 	this.TextColor = text_color;
// 	this.BgImage = bg_image;
// 	this.Visible = false;	

// 	var Padding = {
// 		LEFT : 6,
// 		TOP : 2,
// 		RIGHT : 6,
// 		BUTTOM : 2
// 	}

// 	this.Paint = function(gr) {
// 		if (!this.Visible) return;
			
// 		var tw = gr.CalcTextWidth(this.Text, this.Font);
// 		var th = gr.CalcTextHeight(this.Text, this.Font);
// 		this.Width = tw + Padding.LEFT + Padding.RIGHT;
// 		this.Height = th + Padding.TOP + Padding.BUTTOM
// 		gr.DrawImage(this.BgImage, this.X, this.Y, this.Width, this.Height, 0, 0, this.Width, this.Height);
// 		gr.GdiDrawText(this.Text, this.Font, this.TextColor, this.X + Padding.LEFT, this.Y + Padding.TOP, tw, th, TextAlign.CENTER | TextAlign.NOPREFIX);	
// 	}

// 	this.Repaint = function() {
// 		window.RepaintRect(this.X, this.Y, this.Width, this.Height);
// 	}

// 	this.SetText = function(text) {
// 		this.Text = text;
// 		window.RepaintRect(this.X, this.Y, this.Width, this.Height);
// 	}

// 	this.SetColor = function(color){
// 		this.Color = color;
// 		window.RepaintRect(this.X, this.Y, this.Width, this.Height);
// 	}

// 	this.SetImage = function(image){
// 		this.BgImage = image;
// 		window.RepaintRect(this.X, this.Y, this.Width, this.Height);
// 	}

// 	this.SetFont = function(font){
// 		this.Font = font;
// 		window.RepaintRect(this.X, this.Y, this.Width, this.Height);
// 	}

// 	this.Activate = function() {
// 		this.Visible = true;
// 		this.Repaint();
// 	}

// 	this.Deactivate = function() {
// 		this.Visible = false;
// 		this.Repaint();
// 	}
// }


/**************************************
 * 标签
 *************************************/
function Label(x, y, w, h, text, font, color, textAlign) {
	this.X = x;
	this.Y = y;
	this.Width = w;
	this.Height = h;
	this.Text = text;
	this.Color = color;
	this.Font = font;
	this.Align = textAlign;

	this.SetText = function(text) {
		this.Text = text;
		window.RepaintRect(this.X, this.Y, this.Width, this.Height);
	}

	this.SetColor = function(color){
		this.Color = color;
		window.RepaintRect(this.X, this.Y, this.Width, this.Height);
	}

	this.SetFont = function(font){
		this.Font = font;
		window.RepaintRect(this.X, this.Y, this.Width, this.Height);
	}

	this.Repaint = function() {
		window.RepaintRect(this.X, this.Y, this.Width, this.Height);
	}

	this.Paint = function(gr) {
		gr.GdiDrawText(this.Text, this.Font, this.Color, this.X, this.Y, this.Width, this.Height, this.Align|TextAlign.NOPREFIX |TextAlign.END_ELLIPSIS);
	}
}
/**************************************
 * DPI
 *************************************/

 function DPI(){
	 this.Dpi = 1;

	 this.GetScale= function (value){
		 return Math.floor(value*zdpi);	
		}
 }



 /**************************************
 * 公共方法
 *************************************/
function GetTextHeight(text, font){
	var w = 100;		
	var image = gdi.CreateImage(w, w);
    var gb = image.GetGraphics();
    var height = gb.CalcTextHeight(text, font);	
	image.ReleaseGraphics(gb);
    CollectGarbage(); 
    return height;
}

function GetTextWidth(text, font){
	var w = 100;	
	var image = gdi.CreateImage(w, w);
    var gb = image.GetGraphics();
    var width = gb.CalcTextWidth(text, font);	
	image.ReleaseGraphics(gb);
    CollectGarbage(); 
    return width;
}


function Rect(){
	this.X = 0;
	this.Y = 0;
	this.Width = 0;
	this.Height = 0;
}